@@ -102,11 +102,13 @@ GdkPixbuf* fm_pixbuf_from_icon(FmIcon* icon, int size)
         char* str = g_icon_to_string(icon->gicon);
         g_debug("unable to load icon %s", str);
         /* pix = NULL; */
+#if 0
         if(g_strcmp0(str, "folder-locked") == 0)
             pix = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(), "folder",
                     size, GTK_ICON_LOOKUP_USE_BUILTIN|GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
             /* FIXME: create locked icon from "folder" one */
         else
+#endif
             pix = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(), "unknown",
                     size, GTK_ICON_LOOKUP_USE_BUILTIN|GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
         if(G_LIKELY(pix))
